import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Thread;
import java.util.ArrayList;

class AmaViewer extends DisplayFrame
{
	protected WaveMaker		m_wave;
	protected int radius;

	protected boolean	m_pause;
	protected boolean	m_step;
	protected boolean	m_ff;

	protected int 		m_frameNum = 0;

	static final long serialVersionUID = 1;

	protected HexGrid	m_hist;
	protected HexGrid	m_hist2;
	protected HexGrid	m_timer;
	protected HexGrid	m_timer2;

	public AmaViewer(String str) throws IOException
	{
		super("Amacrine wave viewer - "+str, 20, 50, 1440, 400);	
		m_pause = false;
		//m_pause = true;
		m_step = false;
		m_ff = false;

		m_wave = new WaveMaker(str);
		System.out.println(m_wave.header());
		radius = m_wave.radius();

		m_hist = new HexGrid(radius);
		m_hist2 = new HexGrid(radius);
		m_timer = new HexGrid(radius);
		m_timer2 = new HexGrid(radius);
		for (int y=-radius; y<=radius; y++)
		{
			for (int x=-radius; x<=radius; x++)
			{
				if (!HexGrid.inGrid(x, y, radius))
					continue;

				m_hist.set(x, y, new Integer(0));
				m_hist2.set(x, y, new Integer(0));
				m_timer.set(x, y, new Integer(-200));
				m_timer2.set(x, y, new Integer(-200));
			}
		}
	}

	// parse file on the fly and display image
	// run at ~10fps during activity and ~100fps between waves
	public void runVideo()	throws IOException
	{
		boolean running = true;
		while (running)
		{
			int res = m_wave.nextFrame();
			if (res < 0)
				break;

			m_frameNum++;
			drawImage();

			do
			{
				if (m_step)
					m_pause = true;
				try
				{
					// can't sleep the full 100ms for 10FPS - need
					//	to take processing time into account
					if (m_ff)
						Thread.sleep(1);	// almost as fast as possible
						//Thread.sleep(2);	// almost as fast as possible
					else
						Thread.sleep(50);	
						//Thread.sleep(70);	// ~10 FPS	(realtime)
						//Thread.sleep(100);	// 10 FPS	(realtime)
				}
				catch (InterruptedException ie) { ; }
			} while (m_pause);
		}
		m_wave.close();
		System.out.println("done");
	}

	static final int	BUTTON_WIDTH = 24;
	static final int	BUTTON_HEIGHT = 24;
	static final int	BUTTON_TOP =  5;	
	static final int	BUTTON_PLAY_LEFT	= 10;
	static final int	BUTTON_STEP_LEFT	= 10 + (15+BUTTON_WIDTH);
	static final int	BUTTON_10X_LEFT		= 10 + 2*(15+BUTTON_WIDTH);

	public void hookMouseClicked(MouseEvent e)
	{
		int x = e.getX();
		int y = e.getY();
		boolean handled = false;

		if ((y > BUTTON_TOP) && (y < (BUTTON_TOP+BUTTON_HEIGHT)))
		{
			int xp;

			xp = x - BUTTON_PLAY_LEFT;
			if ((xp > 0) && (xp < BUTTON_WIDTH))
			{
				if (m_pause)
					m_pause = false;
				else
					m_pause = true;
				m_step = false;
				if (m_pause)
					drawImage();	// update dropped otherwise

				handled = true;
			}
			
			xp = x - BUTTON_STEP_LEFT;
			if ((xp > 0) && (xp < BUTTON_WIDTH))
			{
				m_step = true;
				m_pause = false;
				handled = true;
			}
			
			xp = x - BUTTON_10X_LEFT;
			if ((xp > 0) && (xp < BUTTON_WIDTH))
			{
				m_ff = !m_ff;
				drawImage();
				handled = true;
			}
		}
//		else
//		{
//			System.out.println("click at "+x+","+y);
//			int py = (y - 9)/2 - RGC_RADIUS;
//			int px = (x - 225)/2 - RGC_RADIUS - py/2;
//			System.out.println("estimated coords - x="+px+" y="+py);
//		}
	}
	
	public void drawButtons(Graphics gc)
	{
		final int wid = BUTTON_WIDTH;
		final int hei = BUTTON_HEIGHT;

		int py = BUTTON_TOP;
		int px;

		// pause/play
		px = BUTTON_PLAY_LEFT;
		gc.setColor(Color.black);
		gc.drawRect(px, py, wid, hei);
		if (m_pause || m_step)
		{
			for (int i=0; i<10; i++)
			{
				gc.drawLine(px+18-i, py+12-i, px+18-i, py+12+i);
			}
		}
		else
		{
			gc.drawLine(px+9, py+3, px+9, py+hei-3);
			gc.drawLine(px+10, py+3, px+10, py+hei-3);
			gc.drawLine(px+14, py+3, px+14, py+hei-3);
			gc.drawLine(px+15, py+3, px+15, py+hei-3);
		}
		
		// step
		px = BUTTON_STEP_LEFT;
		gc.drawRect(px, py, wid, hei);
		gc.fillRect(px+10, py+10, 4, 4);

		// fast forward
		px = BUTTON_10X_LEFT;
		gc.drawRect(px, py, wid, hei);
		if (m_ff)
		{
			gc.drawLine(px+5, py+hei/2-5, px+5, py+hei/2+5);
			gc.drawRect(px+8, py+hei/2-5, 5, 10);
			gc.drawLine(px+15, py+hei/2, px+20, py+hei/2+5);
			gc.drawLine(px+20, py+hei/2, px+15, py+hei/2+5);
		}
		else
		{
			gc.drawLine(px+9, py+hei/2-5, px+9, py+hei/2+5);
			gc.drawLine(px+13, py+hei/2, px+18, py+hei/2+5);
			gc.drawLine(px+18, py+hei/2, px+13, py+hei/2+5);
		}
	}

	public void drawImage()
	{
		Image img = getImage();
		Graphics gc = img.getGraphics();

		// fill screen to background color
		gc.setColor(Color.white);
		//gc.setColor(new Color(64, 64, 64));
		gc.fillRect(0, 0, m_width, m_height);

		// draw hard boundary
		gc.setColor(Color.blue);
		for (int y=-radius-1; y<=radius+1; y++)
		{
			double xInset = radius + y/2.0;

			for (int x=-radius-1; x<=radius+1; x++)
			{
				int ring = HexGrid.calcRingFromHexSpace(x, y);
				if (ring == radius+1)
				{
					int xx = (int) (     3 * (xInset + x));
					int yy = (int) (50 + 3 * (radius + y));
					gc.fillRect((int)   5 + xx, yy, 3, 3);
					gc.fillRect((int) 365 + xx, yy, 3, 3);
//					gc.fillRect((int) 725 + xx, yy, 3, 3);
				}
			}
		}
		// draw flexible boundary
		gc.setColor(Color.blue);
		ArrayList boundary= m_wave.boundaryList();
		for (int i=0; i<boundary.size(); i++)
		{
			Point pt = (Point) boundary.get(i);
			int x = pt.x;
			int y = pt.y;
			double xInset = radius + y/2.0;
			gc.fillRect((int) (  5 + 3*(xInset+x)), 50 + 3*(radius + y), 3, 3);
			gc.fillRect((int) (365 + 3*(xInset+x)), 50 + 3*(radius + y), 3, 3);
//			gc.fillRect((int) (725 + 3*(xInset+x)), 50 + 3*(radius + y), 3, 3);
		}
		// draw center
		ArrayList cells = m_wave.amaList();
		for (int i=0; i<cells.size(); i++)
		{
			Point pt = (Point) cells.get(i);
			int x = pt.x;
			int y = pt.y;
			double xInset = radius + y/2.0;

			// raw amacrine activity
			Pixel pix = m_wave.pixel(x, y);
			if (pix.amacrine == 0)
				gc.setColor(Color.white);
			else if (pix.amacrine == 1)
				gc.setColor(new Color(0, 255, 0));
			else if (pix.amacrine == 2)
				gc.setColor(new Color(255, 0, 0));
			else if (pix.amacrine == 3)
				gc.setColor(new Color(0, 255, 0));
//if ((x == 18) && (y == -11)) 
//	gc.setColor(Color.black);
//else if ((x == 7) && (y == -5)) 
//	gc.setColor(Color.black);
//else if ((x ==  9) && (y ==  1)) 
//	gc.setColor(Color.black);
//else if ((x ==  9) && (y == 12)) 
//	gc.setColor(Color.black);
			gc.fillRect((int) (5+3*(xInset+x)), 50+3*(radius+y), 3, 3);

			// simulated calcium response
			int sh = (int) ((1.0 - pix.calcium) * 255);
//				if (i < 128)
//					gc.setColor(new Color(255, 0, 0));
//				else
				gc.setColor(new Color(sh, sh, sh));
//if ((x == 18) && (y == -11)) 
//	gc.setColor(Color.red);
//else if ((x == 7) && (y == -5)) 
//	gc.setColor(Color.red);
//else if ((x ==  9) && (y ==  1)) 
//	gc.setColor(Color.red);
//else if ((x ==  9) && (y == 12)) 
//	gc.setColor(Color.red);
			gc.fillRect((int) (365+3*(xInset+x)), 50+3*(radius+y), 3, 3);

			// progression of wave in time
			int hist = ((Integer) m_hist.get(x, y)).intValue();
			int when = ((Integer) m_timer.get(x, y)).intValue();
			if (pix.calcium > StatMaker.THRESH)
			{
				if (when != 0)
				{
					hist = (m_frameNum/5) % 6;
					m_hist.set(x, y, new Integer(hist));
					when = 0;
					m_timer.set(x, y, new Integer(when));
				}
			}
			else if (pix.calcium < StatMaker.THRESH_LOW)
			{
				if (when == 0)
				{
					m_timer.set(x, y, new Integer(m_frameNum));
				}
			}
			if ((when == 0) || ((m_frameNum - when) < 150))
			{
				if (when > 0)
					when = m_frameNum - when;
				int shade = (int) (255.0 * when / 150);
				if (hist == 0)		// red
					gc.setColor(new Color(shade, 255, 255));
				else if (hist == 1)	// aqua
					gc.setColor(new Color(255, shade, shade));
				else if (hist == 2)	// yellow
					gc.setColor(new Color(shade, shade, 255));
				else if (hist == 3)	// blue
					gc.setColor(new Color(255, 255, shade));
				else if (hist == 4)	// purple
					gc.setColor(new Color(shade, 255, shade));
				else if (hist == 5)	// green
					gc.setColor(new Color(255, shade, 255));
			}
			else
				gc.setColor(new Color(224, 224, 224));
			gc.fillRect((int) (725+3*(xInset+x)), 50+3*(radius+y), 3, 3);

			// progression of wave in time
			hist = ((Integer) m_hist2.get(x, y)).intValue();
			when = ((Integer) m_timer2.get(x, y)).intValue();
			pix = m_wave.pixel(x, y);
			if (pix.amacrine > 0)
			{
				if (when != 0)
				{
					hist = (m_frameNum/5) % 6;
					m_hist2.set(x, y, new Integer(hist));
					when = 0;
					m_timer2.set(x, y, new Integer(when));
				}
			}
			else 
			{
				if (when == 0)
				{
					m_timer2.set(x, y, new Integer(m_frameNum));
				}
			}

			if ((when == 0) || ((m_frameNum - when) < 150))
			{
				if (when > 0)
					when = m_frameNum - when;
				int shade = (int) (255.0 * when / 150);
				if (hist == 0)		// red
					gc.setColor(new Color(shade, 255, 255));
				else if (hist == 1)	// aqua
					gc.setColor(new Color(255, shade, shade));
				else if (hist == 2)	// yellow
					gc.setColor(new Color(shade, shade, 255));
				else if (hist == 3)	// blue
					gc.setColor(new Color(255, 255, shade));
				else if (hist == 4)	// purple
					gc.setColor(new Color(shade, 255, shade));
				else if (hist == 5)	// green
					gc.setColor(new Color(255, shade, 255));
			}
			else
				gc.setColor(new Color(224, 224, 224));
			gc.fillRect((int) (1085+3*(xInset+x)), 50+3*(radius+y), 3, 3);
		}

		drawButtons(gc);

		int frameTime = m_wave.frameTime();
		String time = ((int) frameTime / 1000) + "." + (frameTime % 1000);
		gc.drawString(time, BUTTON_TOP + 6*BUTTON_WIDTH, 
					BUTTON_TOP + BUTTON_HEIGHT);

		setImage(img);
		requestDataUpdate();
	}

	///////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////
	public static void main(String[] args)
	{
		try
		{
			int num = -1;
			String name = "../amacrine_1.dat";
			if (args.length >= 1)
			{
				num = Integer.parseInt(args[0]);
				name = "../amacrine_" + num + ".dat";
			}
				
			System.out.println("Data file: " + name);
			AmaViewer view1 = new AmaViewer(name);
			view1.runVideo();
		}
		catch (IOException ioe)
		{
				ioe.toString();
				ioe.printStackTrace();
		}
	}
}

